@@ -2,7 +2,7 @@
  * File name: audio_output.c
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2018 Iurie Nistor 
+ * Copyright (C) 2018 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -37,6 +37,8 @@ gkick_audio_output_create(struct gkick_audio_output **audio_output, int sample_r
                 return GEONKICK_ERROR;
         }
         (*audio_output)->play         = false;
+        (*audio_output)->start_play   = false;
+        (*audio_output)->decay        = -1;
 	(*audio_output)->enabled      = true;
         (*audio_output)->muted        = false;
         (*audio_output)->solo         = false;
@@ -44,6 +46,7 @@ gkick_audio_output_create(struct gkick_audio_output **audio_output, int sample_r
         (*audio_output)->playing_key  = GEONKICK_ANY_KEY;
         (*audio_output)->midi_channel = GEONKICK_ANY_MIDI_CHANNEL;
         (*audio_output)->sample_rate  = sample_rate;
+        (*audio_output)->note_off     = false;
 
         gkick_buffer_new(&(*audio_output)->updated_buffer,
                          (*audio_output)->sample_rate * GEONKICK_MAX_LENGTH);
@@ -103,12 +106,21 @@ enum geonkick_error
 gkick_audio_output_key_pressed(struct gkick_audio_output *audio_output,
                                struct gkick_note_info *key)
 {
+        if (audio_output->muted)
+                return GEONKICK_OK;
+
         if (key->state == GKICK_KEY_STATE_PRESSED) {
-                audio_output->key = *key;
+                audio_output->key   = *key;
+                audio_output->play  = true;
+                audio_output->decay = -1;
                 gkick_audio_output_swap_buffers(audio_output);
-                gkick_audio_add_playing_buffer_to_ring(audio_output);
+                if (!gkick_audio_output_note_off(audio_output)) {
+                        size_t size = gkick_buffer_size(audio_output->playing_buffer);
+                        gkick_audio_add_playing_buffer_to_ring(audio_output, size);
+                }
         } else {
                 audio_output->key.state = key->state;
+                audio_output->decay     = GEKICK_KEY_RELESE_DECAY_TIME;
         }
 
         return GEONKICK_OK;
@@ -118,24 +130,51 @@ gkick_audio_output_key_pressed(struct gkick_audio_output *audio_output,
 void
 gkick_audio_set_play(struct gkick_audio_output *audio_output)
 {
+        if (audio_output->muted)
+                return;
+
         struct gkick_note_info key;
-        key.channel     = GEONKICK_ANY_MIDI_CHANNEL;
-        key.note_number = 69;
-        key.velocity    = 127;
-        key.state       = GKICK_KEY_STATE_PRESSED;
-        audio_output->play = false;
+        key.channel        = GEONKICK_ANY_MIDI_CHANNEL;
+        key.note_number    = GEONKICK_ANY_KEY;
+        key.velocity       = 127;
+        key.state          = GKICK_KEY_STATE_PRESSED;
         gkick_audio_output_key_pressed(audio_output, &key);
 }
 
+gkick_real gkick_audio_get_decay_val(struct gkick_audio_output *audio_output)
+{
+        gkick_real decay_val = 1.0f;
+        int release_time = GEKICK_KEY_RELESE_DECAY_TIME;
+        if (audio_output->key.state == GKICK_KEY_STATE_RELEASED) {
+                audio_output->decay--;
+                if (audio_output->decay < 0) {
+                        audio_output->play = false;
+                        decay_val = 0.0f;
+                        audio_output->decay = -1;
+                } else {
+                        decay_val = - 1.0f * ((gkick_real)(release_time - audio_output->decay)
+                                              / release_time) + 1.0f;
+                }
+        }
+        return decay_val;
+}
+
 
 void
-gkick_audio_add_playing_buffer_to_ring(struct gkick_audio_output *audio_output)
+gkick_audio_add_playing_buffer_to_ring(struct gkick_audio_output *audio_output,
+                                       size_t size)
 {
-        gkick_buffer_reset(audio_output->playing_buffer);
+        if (!audio_output->play)
+                return;
+
         size_t i = 0;
         gkick_real factor = gkick_audio_output_tune_factor(audio_output->key.note_number);
-        factor = geonkick_clamp(factor, 0.5f, 2.0f);
-        while (!gkick_buffer_is_end(audio_output->playing_buffer)) {
+        while (i < size) {
+                if (gkick_buffer_is_end(audio_output->playing_buffer)) {
+                        audio_output->play = false;
+                        break;
+                }
+
                 gkick_real val;
                 if (audio_output->tune) {
                         val = gkick_buffer_stretch_get_next(audio_output->playing_buffer,
@@ -143,7 +182,9 @@ gkick_audio_add_playing_buffer_to_ring(struct gkick_audio_output *audio_output)
                 } else {
                         val = gkick_buffer_get_next(audio_output->playing_buffer);
                 }
-                val *= (gkick_real)audio_output->key.velocity / 127;
+
+                gkick_real decay_val = gkick_audio_get_decay_val(audio_output);
+                val *= decay_val * ((gkick_real)audio_output->key.velocity / 127);
                 ring_buffer_add_value(audio_output->ring_buffer, i++, val);
         }
 }
@@ -151,14 +192,15 @@ gkick_audio_add_playing_buffer_to_ring(struct gkick_audio_output *audio_output)
 enum geonkick_error
 gkick_audio_output_play(struct gkick_audio_output *audio_output)
 {
-        audio_output->play = true;
+        audio_output->start_play = true;
         return GEONKICK_OK;
 }
 
 gkick_real
 gkick_audio_output_tune_factor(int note_number)
 {
-        return exp2f((gkick_real)(note_number - 69) / 12.0f);
+        gkick_real factor = exp2f((gkick_real)(note_number - 69) / 12.0f);
+        return GKICK_CLAMP(factor, 0.5f, 2.0f);
 }
 
 void gkick_audio_output_lock(struct gkick_audio_output *audio_output)
@@ -261,3 +303,33 @@ gkick_audio_output_get_channel(struct gkick_audio_output *audio_output,
         *channel = audio_output->channel;
         return GEONKICK_OK;
 }
+
+void gkick_audio_get_data(struct gkick_audio_output *audio_output,
+                          gkick_real **data,
+                          gkick_real *leveler,
+                          size_t size)
+{
+        if (gkick_audio_output_note_off(audio_output))
+                gkick_audio_add_playing_buffer_to_ring(audio_output, size);
+        *leveler = ring_buffer_get_cur_data(audio_output->ring_buffer);
+        ring_buffer_get_data(audio_output->ring_buffer,
+                             data[0],
+                             size);
+        gkick_real limiter = (gkick_real)audio_output->limiter / 1000000;
+        for (size_t i = 0; i < size; i++) {
+                data[0][i] *= limiter;
+                data[1][i] = data[0][i];
+        }
+        ring_buffer_next(audio_output->ring_buffer, size);
+}
+
+void gkick_audio_output_enable_note_off(struct gkick_audio_output *audio_output,
+                                 bool enable)
+{
+        audio_output->note_off = enable;
+}
+
+bool gkick_audio_output_note_off(struct gkick_audio_output *audio_output)
+{
+        return audio_output->note_off;
+}
